@@ -11,13 +11,12 @@ require Exporter;
 # Specify a directory for the application schema (and recompile):
 # (override with $ENV{OSPERL_SCHEMA_DB})
 
-sub SCHEMA_DBDIR() { 'grbat1p:/export/brain/schema' }
+sub SCHEMA_DBDIR() { 'greeqs092:/export/pinkie/schema' }
 
 
-# Specify a directory for temporary databases (posh, osp_copy, etc):
+# Specify a directory for temporary databases (posh, perltest, etc):
 
-sub TMP_DBDIR() { 'elvis:/opt/os/tmp' }
-#sub TMP_DBDIR() { 'greeqs092:/export/pinkie/tmp' }
+sub TMP_DBDIR() { 'greeqs092:/export/pinkie/tmp' }
 
 
 1;
